# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
# Copyright (c) 2009-2011 Yoshizumi Endo.
#
# This urlget is a modified version of twisted.web.client module.
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""
HTTP Client with proxy support.
"""

import os
from urllib.parse import urlunparse

from ..twisted.web import client
from twisted.internet import reactor
from twisted.web import http

class UrlGetWithProxy(object):

    def __init__(self, proxy=None):
        if proxy is None:
            proxy = os.environ.get('http_proxy') or ""
        self.proxy_host, self.proxy_port = self._parse(proxy)[1:3]
        self.use_proxy = bool(self.proxy_host and self.proxy_port)

    def getPage(self, url, contextFactory=None, *args, **kwargs):
        # FIXME
        if type(url) is str:
            url = url.encode('utf-8')
        # FIXME
        factory = client.HTTPClientFactory(url, *args, **kwargs)
        d = self._urlget(factory, url, contextFactory)
        return d

    def downloadPage(self, url, file, contextFactory=None, *args, **kwargs):
        # FIXME
        if type(url) is str:
            url = url.encode('utf-8')
        # FIXME
        factory = client.HTTPDownloader(url, file, *args, **kwargs)
        d = self._urlget(factory, url, contextFactory)
        return d

    def _urlget(self, factory, url, contextFactory=None):
        scheme, host, port, path = self._parse(url)
        if self.use_proxy and scheme != 'https': # HTTPS proxy isn't supported.
            host, port = self.proxy_host, self.proxy_port
            factory.path = url
        if scheme == 'https':
            from twisted.internet import ssl
            if contextFactory is None:
                contextFactory = ssl.ClientContextFactory()
            reactor.connectSSL(host, port, factory, contextFactory)
        else:
            reactor.connectTCP(host, port, factory)
        return factory.deferred

    def _parse(self, url, defaultPort=None):
        """
        Split the given URL into the scheme, host, port, and path.
                
        @type url: C{str}
        @param url: An URL to parse.
                        
        @type defaultPort: C{int} or C{None}
        @param defaultPort: An alternate value to use as the port if the URL does
        not include one.
    
        @return: A four-tuple of the scheme, host, port, and path of the URL.  All
        of these are C{str} instances except for port, which is an C{int}.
        """
        url = url.strip()
        # FIXME
        if type(url) is str:
            url = url.encode('utf-8')
        # FIXME
        parsed = http.urlparse(url).decode('utf-8')
        scheme = parsed[0]
        path = urlunparse(('', '') + parsed[2:])
    
        if defaultPort is None:
            if scheme == 'https':
                defaultPort = 443
            else:
                defaultPort = 80
    
        host, port = parsed[1], defaultPort
        if ':' in host:
            host, port = host.split(':')
            try:
                port = int(port)
            except ValueError:
                port = defaultPort
    
        if path == '':
            path = '/'
    
        return scheme, host, port, path
